@@ -1,5 +1,5 @@
-/* $Id: procan_main.c,v 1.6 2003/05/21 05:16:38 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2003 */
+/* $Id: procan_main.c,v 1.10 2004/12/11 13:28:05 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 const char copyright[] = "procan by Gerhard Rieger - send bug reports to socat@dest-unreach.org";
@@ -9,7 +9,6 @@ const char copyright[] = "procan by Gerhard Rieger - send bug reports to socat@d
 #include <stdio.h>
 #include "mytypes.h"
 #include "error.h"
-#include "filan.h"
 #include "procan.h"
 
 
@@ -29,7 +28,9 @@ int main(int argc, const char *argv[]) {
    arg1 = argv+1;  --argc;
    while (arg1[0] && (arg1[0][0] == '-')) {
       switch (arg1[0][1]) {
-      case '?': procan_usage(stdout); exit(0);
+#if WITH_HELP
+      case '?': case 'h': procan_usage(stdout); exit(0);
+#endif /* WITH_HELP */
 #if LATER
       case 'V': procan_version(stdout); exit(0);
       case 'l': diag_set(arg1[0][2], &arg1[0][3]); break;
@@ -78,7 +79,7 @@ static void procan_usage(FILE *fd) {
    fputs("      -V     print version information to stdout, and exit\n", fd);
 #endif
 #if WITH_HELP
-   fputs("      -?     print a help text describing command line options\n", fd);
+   fputs("      -?|-h  print a help text describing command line options\n", fd);
 #endif
 #if LATER
    fputs("      -d     increase verbosity (use up to 4 times; 2 are recommended)\n", fd);
