@@ -1,4 +1,4 @@
-/* $Id: socat.c,v 1.87 2004/06/20 21:46:10 gerhard Exp $ */
+/* $Id: socat.c,v 1.90 2004/12/05 12:24:53 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -34,7 +34,18 @@ struct {
    char logopt;		/* y..syslog; s..stderr; f..file; m..mixed */
    bool lefttoright;	/* first addr ro, second addr wo */
    bool righttoleft;	/* first addr wo, second addr ro */
-} socat_opts = { 8192, false, false, 0, {0,500000}, 0, 0, 's', false, false };
+} socat_opts = {
+   8192,	/* bufsiz */
+   false,	/* verbose */
+   false,	/* verbhex */
+   0,		/* timeout */
+   {0,500000},	/* closwait */
+   0,		/* debug */
+   0,		/* strictopts */
+   's',		/* logopt */
+   false,	/* lefttoright */
+   false	/* righttoleft */
+};
 
 void socat_usage(FILE *fd);
 void socat_version(FILE *fd);
@@ -76,8 +87,9 @@ int main(int argc, const char *argv[]) {
       case 'V': socat_version(stdout); Exit(0);
 #if WITH_HELP
       case '?':
+      case 'h':
 	 socat_usage(stdout);
-	 xioopenhelp(stdout, arg1[0][2]=='?' ? arg1[0][3]=='?' ? 2 : 1 : 0);
+	 xioopenhelp(stdout, (arg1[0][2]=='?'||arg1[0][2]=='h') ? (arg1[0][3]=='?'||arg1[0][3]=='h') ? 2 : 1 : 0);
 	 Exit(0);
 #endif /* WITH_HELP */
       case 'd': diag_set('d', NULL); break;
@@ -106,7 +118,7 @@ int main(int argc, const char *argv[]) {
 	    diag_set('u', NULL);
 	    break;
 	 default:
-	    Error1("unknown log option \"%s\"; use option \"-?\" for help", arg1[0]);
+	    Error1("unknown log option \"%s\"; use option \"-h\" for help", arg1[0]);
 	    break;
 	 }
 	 break;
@@ -117,7 +129,7 @@ int main(int argc, const char *argv[]) {
 	 } else {
 	    ++arg1, --argc;
 	    if ((a = *arg1) == NULL) {
-	       Error("option -b requires an argument; use option \"-?\" for help");
+	       Error("option -b requires an argument; use option \"-h\" for help");
 	       socat_usage(stderr); Exit(1);
 	    }
 	 }
@@ -134,7 +146,7 @@ int main(int argc, const char *argv[]) {
 	 } else {
 	    ++arg1, --argc;
 	    if ((a = *arg1) == NULL) {
-	       Error("option -t requires an argument; use option \"-?\" for help");
+	       Error("option -t requires an argument; use option \"-h\" for help");
 	       socat_usage(stderr); Exit(1);
 	    }
 	 }
@@ -154,7 +166,7 @@ int main(int argc, const char *argv[]) {
 	 if (arg1[0][1] == buff[0]) {
 	    break;
 	 }
-	 Error1("unknown option \"%s\"; use option \"-?\" for help", arg1[0]);
+	 Error1("unknown option \"%s\"; use option \"-h\" for help", arg1[0]);
 	 socat_usage(stderr); Exit(1);
       }
       /* the leading "-" might be a form of the first address */
@@ -166,7 +178,7 @@ int main(int argc, const char *argv[]) {
       ++arg1; --argc;
    }
    if (argc != 2) {
-      Error1("exactly 2 addresses required (there are %d); use option \"-?\" for help", argc);
+      Error1("exactly 2 addresses required (there are %d); use option \"-h\" for help", argc);
       /*0 socat_usage(stderr);*/
       Exit(1);
    }
@@ -216,9 +228,9 @@ void socat_usage(FILE *fd) {
    fputs("   options:\n", fd);
    fputs("      -V     print version and feature information to stdout, and exit\n", fd);
 #if WITH_HELP
-   fputs("      -?     print a help text describing command line options and addresses\n", fd);
-   fputs("      -??    like -?, plus a list of all common address option names\n", fd);
-   fputs("      -???    like -?, plus a list of all available address option names\n", fd);
+   fputs("      -h|-?  print a help text describing command line options and addresses\n", fd);
+   fputs("      -hh    like -h, plus a list of all common address option names\n", fd);
+   fputs("      -hhh   like -hh, plus a list of all available address option names\n", fd);
 #endif /* WITH_HELP */
    fputs("      -d     increase verbosity (use up to 4 times; 2 are recommended)\n", fd);
 #if WITH_FILAN
@@ -579,16 +591,16 @@ int _socat(void) {
 
       fdi = XIO_GETRDFD(sock1);
       fdo = XIO_GETWRFD(sock1);
-      filan(fdi, stderr);
+      filan_fd(fdi, stderr);
       if (fdo != fdi) {
-	 filan(fdo, stderr);
+	 filan_fd(fdo, stderr);
       }
 
       fdi = XIO_GETRDFD(sock2);
       fdo = XIO_GETWRFD(sock2);
-      filan(fdi, stderr);
+      filan_fd(fdi, stderr);
       if (fdo != fdi) {
-	 filan(fdo, stderr);
+	 filan_fd(fdo, stderr);
       }
 
       diagtxt[0] = exitlevel+'0'; diagtxt[1] = '\0';
