@@ -1,4 +1,4 @@
-/* $Id: sycls.c,v 1.63 2004/09/18 12:12:27 gerhard Exp $ */
+/* $Id: sycls.c,v 1.64 2004/10/24 13:30:25 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -671,6 +671,17 @@ int Chmod(const char *path, mode_t mode) {
    return retval;
 }
 
+#if HAVE_POLL
+/* we only show the first struct pollfd; hope this is enough for most cases. */
+int Poll(struct pollfd *ufds, unsigned int nfds, int timeout) {
+   int result;
+   Debug4("poll({%d, 0x%02hx, }, %u, %d)", ufds[0].fd, ufds[0].events, nfds, timeout);
+   result = poll(ufds, nfds, timeout);
+   Debug2("poll(, {,, 0x%02hx}) -> %d", ufds[0].revents, result);
+   return result;
+}
+#endif /* HAVE_POLL */
+
 /* we only show the first word of the fd_set's; hope this is enough for most
    cases. */
 int Select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
