@@ -1,4 +1,4 @@
-/* $Id: sycls.h,v 1.40 2004/09/21 19:11:24 gerhard Exp $ */
+/* $Id: sycls.h,v 1.41 2004/10/24 13:31:14 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -70,6 +70,7 @@ int Symlink(const char *oldpath, const char *newpath);
 int Readlink(const char *path, char *buf, size_t bufsiz);
 int Chown(const char *path, uid_t owner, gid_t group);
 int Chmod(const char *path, mode_t mode);
+int Poll(struct pollfd *ufds, unsigned int nfds, int timeout);
 int Select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
 	   struct timeval *timeout);
 pid_t Fork(void);
@@ -193,6 +194,7 @@ void Add_history(const char *string);
 #define Readlink(p,b,s) readlink(p,b,s)
 #define Chown(p,o,g) chown(p,o,g)
 #define Chmod(p,m) chmod(p,m)
+#define Poll(u, n, t) poll(u, n, t)
 #define Select(n,r,w,e,t) select(n,r,w,e,t)
 #define Fork() fork()
 #define Waitpid(p,s,o) waitpid(p,s,o)
