@@ -1,4 +1,4 @@
-/* $Id: xioopts.c,v 1.66 2004/06/20 21:18:17 gerhard Exp $ */
+/* $Id: xioopts.c,v 1.70 2004/12/03 21:53:24 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -115,6 +115,9 @@ bool xioopts_ignoregroups;
    binary search! */
 /* NULL terminated */
 const struct optname optionnames[] = {
+#ifdef TCP_ABORT_THRESHOLD  /* HP_UX */
+	IF_TCP    ("abort-threshold",	&opt_tcp_abort_threshold)
+#endif
 #ifdef SO_ACCEPTCONN /* AIX433 */
 	IF_SOCKET ("acceptconn",	&opt_so_acceptconn)
 #endif /* SO_ACCEPTCONN */
@@ -166,6 +169,9 @@ const struct optname optionnames[] = {
 #endif
 #ifdef B3500000
 	IF_TERMIOS("b3500000",	&opt_b3500000)
+#endif
+#ifdef B3600	/* HP-UX */
+	IF_TERMIOS("b3600",	&opt_b3600)
 #endif
 	IF_TERMIOS("b38400",	&opt_b38400)
 #ifdef B4000000
@@ -186,7 +192,13 @@ const struct optname optionnames[] = {
 	IF_TERMIOS("b576000",	&opt_b576000)
 #endif
 	IF_TERMIOS("b600",	&opt_b600)
+#ifdef B7200	/* HP-UX */
+	IF_TERMIOS("b7200",	&opt_b7200)
+#endif
 	IF_TERMIOS("b75",	&opt_b75)
+#ifdef B900	/* HP-UX */
+	IF_TERMIOS("b900",	&opt_b900)
+#endif
 #ifdef B921600
 	IF_TERMIOS("b921600",	&opt_b921600)
 #endif
@@ -233,6 +245,9 @@ const struct optname optionnames[] = {
 	/*IF_NAMED  ("cleanup",	&opt_cleanup)*/
 	IF_TERMIOS("clocal",	&opt_clocal)
 	IF_ANY    ("cloexec",	&opt_cloexec)
+#ifdef TCP_CONN_ABORT_THRESHOLD  /* HP_UX */
+	IF_TCP    ("conn-abort-threshold",	&opt_tcp_conn_abort_threshold)
+#endif
 #ifdef TCP_CORK
 	IF_TCP    ("cork",	&opt_tcp_cork)
 #endif
@@ -303,6 +318,9 @@ const struct optname optionnames[] = {
 	IF_SOCKET ("dontlinger",	&opt_so_dontlinger)
 #endif
 	IF_SOCKET ("dontroute",	&opt_so_dontroute)
+#ifdef VDSUSP	/* HP-UX */
+	IF_TERMIOS("dsusp",	&opt_vdsusp)
+#endif
 #ifdef O_DSYNC
 	IF_OPEN   ("dsync",	&opt_o_dsync)
 #endif
@@ -503,6 +521,9 @@ const struct optname optionnames[] = {
 #ifdef TCP_KEEPIDLE	/* Linux 2.4.0 */
 	IF_TCP    ("keepidle",	&opt_tcp_keepidle)
 #endif
+#ifdef TCP_KEEPINIT	/* OSF1 */
+	IF_TCP    ("keepinit",	&opt_tcp_keepinit)
+#endif
 #ifdef TCP_KEEPINTVL	/* Linux 2.4.0 */
 	IF_TCP    ("keepintvl",	&opt_tcp_keepintvl)
 #endif
@@ -553,6 +574,9 @@ const struct optname optionnames[] = {
 #ifdef TCP_MAXSEG
 	IF_TCP    ("maxseg",	&opt_tcp_maxseg)
 	IF_TCP    ("maxseg-late",	&opt_tcp_maxseg_late)
+#endif
+#ifdef TCP_MD5SUM
+	IF_TCP    ("md5sig",	&opt_tcp_md5sig)
 #endif
 	IF_OPENSSL("method",	&opt_openssl_method)
 	IF_TERMIOS("min",	&opt_vmin)
@@ -604,7 +628,13 @@ const struct optname optionnames[] = {
 	IF_ANY    ("noinherit",		&opt_o_noinherit)
 #endif
 	IF_ANY    ("nonblock",	&opt_nonblock)
+#ifdef TCP_NOOPT
+	IF_TCP    ("noopt",		&opt_tcp_noopt)
+#endif
 	IF_READLINE("noprompt",	&opt_noprompt)
+#ifdef TCP_NOPUSH
+	IF_TCP    ("nopush",	&opt_tcp_nopush)
+#endif
 #ifdef SO_NOREUSEADDR	/* AIX 4.3.3 */
 	IF_SOCKET ("noreuseaddr",	&opt_so_noreuseaddr)
 #endif /* SO_NOREUSEADDR */
@@ -762,6 +792,9 @@ const struct optname optionnames[] = {
 	IF_SOCKET ("passcred",	&opt_so_passcred)
 #endif
 	IF_EXEC   ("path",	&opt_path)
+#ifdef TCP_PAWS	/* OSF1 */
+	IF_TCP    ("paws",	&opt_tcp_paws)
+#endif
 #ifdef SO_PEERCRED
 	IF_SOCKET ("peercred",	&opt_so_peercred)
 #endif
@@ -806,6 +839,10 @@ const struct optname optionnames[] = {
 #if HAVE_PTY
 	IF_EXEC   ("pty",	&opt_pty)
 #endif
+#if HAVE_PTY && HAVE_POLL
+	IF_PTY    ("pty-intervall",	&opt_pty_intervall)
+	IF_PTY    ("pty-wait-slave",	&opt_pty_wait_slave)
+#endif /* HAVE_PTY && HAVE_POLL */
 #ifdef TCP_QUICKACK
 	IF_TCP    ("quickack",	&opt_tcp_quickack)
 #endif
@@ -862,6 +899,12 @@ const struct optname optionnames[] = {
 #endif
 #ifdef O_RSYNC
 	IF_OPEN   ("rsync",	&opt_o_rsync)
+#endif
+#ifdef TCP_SACK_DISABLE
+	IF_TCP    ("sack-disable",	&opt_tcp_sack_disable)
+#endif
+#ifdef TCP_SACKENA	/* OSF1 */
+	IF_TCP    ("sackena",	&opt_tcp_sackena)
 #endif
 	IF_TERMIOS("sane",	&opt_sane)
 #ifdef SO_SECURITY_AUTHENTICATION
@@ -908,6 +951,9 @@ const struct optname optionnames[] = {
 #endif
 	IF_EXEC   ("sighup",	&opt_sighup)
 	IF_EXEC   ("sigint",	&opt_sigint)
+#ifdef TCP_SIGNATURE_ENABLE
+	IF_TCP    ("signature-enable",	&opt_tcp_signature_enable)
+#endif
 	IF_EXEC   ("sigquit",	&opt_sigquit)
 #ifdef SIOCSPGRP
 	IF_SOCKET ("siocspgrp",	&opt_siocspgrp)
@@ -1053,6 +1099,12 @@ const struct optname optionnames[] = {
 	IF_TERMIOS("tabdly",	&opt_tabdly)
 #endif
 	IF_TERMIOS("tandem",	&opt_ixoff)
+#ifdef TCP_ABORT_THRESHOLD  /* HP_UX */
+	IF_TCP    ("tcp-abort-threshold",	&opt_tcp_abort_threshold)
+#endif
+#ifdef TCP_CONN_ABORT_THRESHOLD  /* HP_UX */
+	IF_TCP    ("tcp-conn-abort-threshold",	&opt_tcp_conn_abort_threshold)
+#endif
 #ifdef TCP_CORK
 	IF_TCP    ("tcp-cork",	&opt_tcp_cork)
 #endif
@@ -1068,6 +1120,9 @@ const struct optname optionnames[] = {
 #ifdef TCP_KEEPIDLE	/* Linux 2.4.0 */
 	IF_TCP    ("tcp-keepidle",	&opt_tcp_keepidle)
 #endif
+#ifdef TCP_KEEPINIT	/* OSF1 */
+	IF_TCP    ("tcp-keepinit",	&opt_tcp_keepinit)
+#endif
 #ifdef TCP_KEEPINTVL	/* Linux 2.4.0 */
 	IF_TCP    ("tcp-keepintvl",	&opt_tcp_keepintvl)
 #endif
@@ -1078,21 +1133,45 @@ const struct optname optionnames[] = {
 	IF_TCP    ("tcp-maxseg",	&opt_tcp_maxseg)
 	IF_TCP    ("tcp-maxseg-late",	&opt_tcp_maxseg_late)
 #endif
+#ifdef TCP_MD5SIG
+	IF_TCP    ("tcp-md5sig",	&opt_tcp_md5sig)
+#endif
 #ifdef TCP_NODELAY
 	IF_TCP    ("tcp-nodelay",	&opt_tcp_nodelay)
 #endif
+#ifdef TCP_NOOPT
+	IF_TCP    ("tcp-noopt",		&opt_tcp_noopt)
+#endif
+#ifdef TCP_NOPUSH
+	IF_TCP    ("tcp-nopush",	&opt_tcp_nopush)
+#endif
+#ifdef TCP_PAWS	/* OSF1 */
+	IF_TCP    ("tcp-paws",		&opt_tcp_paws)
+#endif
 #ifdef TCP_QUICKACK
 	IF_TCP    ("tcp-quickack",	&opt_tcp_quickack)
 #endif
 #ifdef TCP_RFC1323
 	IF_TCP    ("tcp-rfc1323",	&opt_tcp_rfc1323)
 #endif
+#ifdef TCP_SACK_DISABLE
+	IF_TCP    ("tcp-sack-disable",	&opt_tcp_sack_disable)
+#endif
+#ifdef TCP_SACKENA	/* OSF1 */
+	IF_TCP    ("tcp-sackena",	&opt_tcp_sackena)
+#endif
+#ifdef TCP_SIGNATURE_ENABLE
+	IF_TCP    ("tcp-signature-enable",	&opt_tcp_signature_enable)
+#endif
 #ifdef TCP_STDURG
 	IF_TCP    ("tcp-stdurg",	&opt_tcp_stdurg)
 #endif
 #ifdef TCP_SYNCNT	/* Linux 2.4.0 */
 	IF_TCP    ("tcp-syncnt",	&opt_tcp_syncnt)
 #endif
+#ifdef TCP_TSOPTENA	/* OSF1 */
+	IF_TCP    ("tcp-tsoptena",	&opt_tcp_tsoptena)
+#endif
 #ifdef TCP_WINDOW_CLAMP	/* Linux 2.4.0 */
 	IF_TCP    ("tcp-window-clamp",	&opt_tcp_window_clamp)
 #endif
@@ -1113,6 +1192,9 @@ const struct optname optionnames[] = {
 	IF_ANY    ("truncate",	&opt_ftruncate64)
 #else
 	IF_ANY    ("truncate",	&opt_ftruncate32)
+#endif
+#ifdef TCP_TSOPTENA	/* OSF1 */
+	IF_TCP    ("tsoptena",	&opt_tcp_tsoptena)
 #endif
 	IF_IP     ("ttl",	&opt_ip_ttl)
 	IF_SOCKET ("type",	&opt_so_type)
@@ -1135,6 +1217,9 @@ const struct optname optionnames[] = {
 	IF_ANY    ("user-late",	&opt_user_late)
 #ifdef VDISCARD
 	IF_TERMIOS("vdiscard",	&opt_vdiscard)
+#endif
+#ifdef VDSUSP	/* HP-UX */
+	IF_TERMIOS("vdsusp",	&opt_vdsusp)
 #endif
 	IF_TERMIOS("veof",	&opt_veof)
 	IF_TERMIOS("veol",	&opt_veol)
@@ -1168,6 +1253,10 @@ const struct optname optionnames[] = {
 #ifdef VWERASE
 	IF_TERMIOS("vwerase",	&opt_vwerase)
 #endif
+#if HAVE_PTY && HAVE_POLL
+	IF_PTY    ("wait-slave",	&opt_pty_wait_slave)
+	IF_PTY    ("waitslave",	&opt_pty_wait_slave)
+#endif /* HAVE_PTY && HAVE_POLL */
 #ifdef VWERASE
 	IF_TERMIOS("werase",	&opt_vwerase)
 #endif
@@ -1366,7 +1455,7 @@ int parseopts(char *a, unsigned int groups, struct opt **opts) {
 	 Info2("setting option \"%s\" to %u", ent->desc->defname,
 	       (*opts)[i].value.u_long);
 	 break;
-#if HAVE_TYPE_LONGLONG && HAVE_STRTOLL
+#if HAVE_TYPE_LONGLONG
       case TYPE_LONGLONG:
 #  if HAVE_STAT64 && defined(HAVE_BASIC_OFF64_T) && HAVE_BASIC_OFF64_T==7
       case TYPE_OFF64:
@@ -1374,13 +1463,18 @@ int parseopts(char *a, unsigned int groups, struct opt **opts) {
 	 if (optr != '=') {
 	    (*opts)[i].value.u_longlong = 1;
 	 } else {
+#  if HAVE_STRTOLL
 	    slonglongval = strtoll(a2, &a2, 0);
+#  else
+	    /* in this case, input value range is limited */
+	    slonglongval = strtol(a2, &a2, 0);
+#  endif /* HAVE_STRTOLL */
 	    (*opts)[i].value.u_longlong = slonglongval;
 	 }
 	 Info2("setting option \"%s\" to %u", ent->desc->defname,
 	       (*opts)[i].value.u_longlong);
 	 break;
-#endif /* HAVE_TYPE_LONGLONG && HAVE_STRTOLL */
+#endif /* HAVE_TYPE_LONGLONG */
       case TYPE_UIDT:
 	 if (a2 == NULL) { Error1("option \"%s\": value required", a0);
 	    continue; }
@@ -1696,6 +1790,30 @@ int retropt(struct opt *opts, int optcode, union integral *result) {
 }
 #endif
 
+static struct opt *xio_findopt(struct opt *opts, int optcode) {
+   struct opt *opt = opts;
+
+   while (opt->desc != ODESC_END) {
+      if (opt->desc != ODESC_DONE && opt->desc->optcode == optcode) {
+	 return opt;
+      }
+      ++opt;
+   }
+   return NULL;
+}
+
+int retropt_timespec(struct opt *opts, int optcode, struct timespec *result) {
+   struct opt *opt;
+
+   if (!(opt = xio_findopt(opts, optcode))) {
+      return -1;
+   }
+   *result = opt->value.u_timespec;
+   opt->desc = ODESC_DONE;
+   return 0;
+}
+      
+   
 /* Looks for the first option of type <optcode>. If the option is found,
    this function stores its bool value in *result, "consumes" the
    option, and returns 0.
