@@ -1,4 +1,4 @@
-/* $Id: xioopts.h,v 1.45 2004/06/20 21:17:39 gerhard Exp $ */
+/* $Id: xioopts.h,v 1.49 2004/12/03 21:53:24 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -157,10 +157,13 @@ enum e_optcode {
    OPT_B200,		/* termios.c_cflag */
    OPT_B300,		/* termios.c_cflag */
    OPT_B600,		/* termios.c_cflag */
+   OPT_B900,		/* termios.c_cflag - HP-UX */
    OPT_B1200,		/* termios.c_cflag */
    OPT_B1800,		/* termios.c_cflag */
    OPT_B2400,		/* termios.c_cflag */
+   OPT_B3600,		/* termios.c_cflag - HP-UX */
    OPT_B4800,		/* termios.c_cflag */
+   OPT_B7200,		/* termios.c_cflag - HP-UX */
    OPT_B9600,		/* termios.c_cflag */
    OPT_B19200,		/* termios.c_cflag */
    OPT_B38400,		/* termios.c_cflag */
@@ -450,6 +453,8 @@ enum e_optcode {
    OPT_PTMX,
 #endif
    OPT_PTY,
+   OPT_PTY_INTERVALL,
+   OPT_PTY_WAIT_SLAVE,
    OPT_RANGE,		/* restrict client socket address */
    OPT_RAW,		/* termios */
    OPT_RETRY,
@@ -588,6 +593,8 @@ enum e_optcode {
    OPT_TABDLY,		/* termios.c_oflag */
 #endif
    OPT_TCPWRAPPERS,	/* libwrap */
+   OPT_TCP_ABORT_THRESHOLD,	/* HP-UX */
+   OPT_TCP_CONN_ABORT_THRESHOLD,	/* HP-UX */
 #ifdef TCP_CORK
    OPT_TCP_CORK,
 #endif
@@ -603,6 +610,7 @@ enum e_optcode {
 #ifdef TCP_KEEPIDLE
    OPT_TCP_KEEPIDLE,	/* Linux 2.4.0 */
 #endif
+   OPT_TCP_KEEPINIT,	/* OSF1 */
 #ifdef TCP_KEEPINTVL
    OPT_TCP_KEEPINTVL,	/* Linux 2.4.0 */
 #endif
@@ -613,21 +621,29 @@ enum e_optcode {
    OPT_TCP_MAXSEG,
    OPT_TCP_MAXSEG_LATE,
 #endif
+   OPT_TCP_MD5SIG,	/* FreeBSD */
 #ifdef TCP_NODELAY
    OPT_TCP_NODELAY,
 #endif
+   OPT_TCP_NOOPT,	/* FreeBSD */
+   OPT_TCP_NOPUSH,	/* FreeBSD */
+   OPT_TCP_PAWS,	/* OSF1 */
 #ifdef TCP_QUICKACK
    OPT_TCP_QUICKACK,	/* Linux 2.4 */
 #endif
 #ifdef TCP_RFC1323
    OPT_TCP_RFC1323,	/* AIX 4.3.3 */
 #endif
+   OPT_TCP_SACKENA,	/* OSF1 */
+   OPT_TCP_SACK_DISABLE,	/* OpenBSD */
+   OPT_TCP_SIGNATURE_ENABLE,	/* OpenBSD */
 #ifdef TCP_STDURG
    OPT_TCP_STDURG,	/* AIX 4.3.3; Linux: see man 7 tcp */
 #endif
 #ifdef TCP_SYNCNT
    OPT_TCP_SYNCNT,	/* Linux 2.4.0 */
 #endif
+   OPT_TCP_TSOPTENA,	/* OSF1 */
 #ifdef TCP_WINDOW_CLAMP
    OPT_TCP_WINDOW_CLAMP,	/* Linux 2.4.0 */
 #endif
@@ -643,6 +659,7 @@ enum e_optcode {
 #ifdef VDISCARD
    OPT_VDISCARD,	/* termios.c_cc */
 #endif
+   OPT_VDSUSP,		/* termios.c_cc - HP-UX */
    OPT_VEOF,		/* termios.c_cc */
    OPT_VEOL,		/* termios.c_cc */
    OPT_VEOL2,		/* termios.c_cc */
@@ -747,6 +764,7 @@ extern int retropt_long(struct opt *opts, int optcode, long *result);
 extern int retropt_ulong(struct opt *opts, int optcode, unsigned long *result);
 extern int retropt_flag(struct opt *opts, int optcode, flags_t *result);
 extern int retropt_string(struct opt *opts, int optcode, char **result);
+extern int retropt_timespec(struct opt *opts, int optcode, struct timespec *result);
 extern int retropt_bind(struct opt *opts,
 		 int af,
 		 const char *protoname,	/* for service resolution */
