@@ -1,5 +1,5 @@
-/* $Id: xioshutdown.c,v 1.16 2003/12/23 16:51:08 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2003 */
+/* $Id: xioshutdown.c,v 1.17 2004/09/18 12:14:12 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended shutdown function */
@@ -8,6 +8,12 @@
 #include "xiosysincludes.h"
 #include "xioopen.h"
 
+static pid_t socat_kill_pid;	/* here we pass the pid to be killed in sighandler */
+
+static void signal_kill_pid(int dummy) {
+   Notice("SIGALRM while waiting for w/o child process to die, killing it now");
+   Kill(socat_kill_pid, SIGTERM);
+}
 
 int xioshutdown(xiofile_t *sock, int how) {
    int result = 0;
@@ -61,12 +67,38 @@ int xioshutdown(xiofile_t *sock, int how) {
       }
       
 #if WITH_SOCKET
-   } else if (sock->stream.howtoend == END_SHUTDOWN ||
-	      sock->stream.howtoend == END_SHUTDOWN_KILL) {
+   } else if (sock->stream.howtoend == END_SHUTDOWN) {
       if ((result = Shutdown(sock->stream.fd, how)) < 0) {
 	 Warn3("shutdown(%d, %d): %s",
 	       sock->stream.fd, how, strerror(errno));
       }
+   } else if (sock->stream.howtoend == END_SHUTDOWN_KILL) {
+      if ((result = Shutdown(sock->stream.fd, how)) < 0) {
+	 Warn3("shutdown(%d, %d): %s",
+	       sock->stream.fd, how, strerror(errno));
+      }
+      if (sock->stream.flags == O_WRONLY) {
+	 /* the child process might want to flush some data before terminating
+	    */
+	 int status = 0;
+
+	 /* we wait for the child process to die, but to prevent timeout
+	    we raise an alarm after some time.
+	    NOTE: the alarm does not terminate waitpid() on Linux/glibc (BUG?),
+	    therefore we have to do the kill in the signal handler */
+	 Signal(SIGALRM, signal_kill_pid);
+	 socat_kill_pid = sock->stream.para.exec.pid;
+#if HAVE_SETITIMER
+	 /*! with next feature release, we get usec resolution and an option */
+#else
+	 Alarm(1 /*! sock->stream.para.exec.waitdie */);
+#endif /* !HAVE_SETITIMER */
+	 if (Waitpid(sock->stream.para.exec.pid, &status, 0) < 0) {
+	    Warn3("waitpid("F_pid", %p, 0): %s",
+		  sock->stream.para.exec.pid, &status, strerror(errno));
+	 }
+	 Alarm(0);
+      }
    } else if (sock->stream.dtype == DATA_RECV) {
       if (how >= 1) {
 	 if (Close(sock->stream.fd) < 0) {
